package org.jboss.security.srp;

package .tmp;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import javax.naming.InitialContext;
import javax.naming.Name;
import org.jboss.naming.NonSerializableFactory;
import org.jboss.security.srp.SRPVerifierStoreServiceMBean;
import org.jboss.security.srp.SerialObjectStore;
import org.jboss.system.ServiceMBeanSupport;

public class SRPVerifierStoreService extends ServiceMBeanSupport implements SRPVerifierStoreServiceMBean {
  private SerialObjectStore store;
  
  private String fileName = "SRPVerifierStore.ser";
  
  private String jndiName = "srp/DefaultVerifierSource";
  
  public String getJndiName() {
    return this.jndiName;
  }
  
  public void setJndiName(String jndiName) {
    this.jndiName = jndiName;
  }
  
  public void setStoreFile(String fileName) throws IOException {
    this.fileName = fileName;
    if (this.store != null) {
      File storeFile = new File(fileName);
      this.store.save(storeFile);
    } 
  }
  
  public void addUser(String username, String password) throws IOException {
    try {
      this.store.addUser(username, password);
      save();
      this.log.debug("Added username: " + username);
    } catch (Exception e) {
      this.log.warn("Failed to addUser, username=" + username, e);
    } 
  }
  
  public void delUser(String username) throws IOException {
    this.store.delUser(username);
    this.log.debug("Added username: " + username);
    save();
  }
  
  public String getName() {
    return "SRPVerifierStoreService";
  }
  
  public void initService() throws Exception {}
  
  public void startService() throws Exception {
    File storeFile = new File(this.fileName);
    this.store = new SerialObjectStore(storeFile);
    this.log.info("Created SerialObjectStore at: " + storeFile.getAbsolutePath());
    InitialContext ctx = new InitialContext();
    Name name = ctx.getNameParser("").parse(this.jndiName);
    NonSerializableFactory.rebind(name, this.store, true);
  }
  
  private void save() throws IOException {
    if (this.store != null) {
      File storeFile = new File(this.fileName);
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      URL url = loader.getResource(this.fileName);
      if (url == null) {
        String parent = storeFile.getParent();
        if (parent != null) {
          url = loader.getResource(parent);
          if (url != null)
            storeFile = new File(url.getFile(), storeFile.getName()); 
        } 
      } else {
        storeFile = new File(url.getFile());
      } 
      this.store.save(storeFile);
    } 
  }
}
